import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;



public class Problem
{
    static final int VERTEX_MAX = 10000;
    // Instance data
    int num_of_vertices;
    int num_of_edges;
    int max_degree;
    int min_degree;
    float density;
    float av_degree;
    int declared_edges;

    ArrayList<Integer>[] adjacency_list;
    int adjacency_matrix[][];
    int degree[];
    // vector<pair<int, int> > edges;


    public boolean Edge(int i, int j) {
	return (adjacency_matrix[i][j]==1);
    }
    /**
     * Load data from a file
     * 
     * @param filename
     * @throws FileNotFoundException
     * @throws IOException
     */
    public void loadFromFile(String filename, boolean print_statistics) throws FileNotFoundException, IOException{

        BufferedReader reader = new BufferedReader(new FileReader(filename));
        
	num_of_edges = 0;
	declared_edges = 0;// for doublechecking file definition

        for(String s = reader.readLine();s!=null;s=reader.readLine()){
	    // c comment
            if(s.matches("c.*")){
                continue;
            }
	    // p problem dimensions
            if(s.matches("p.*")){
		String[] tokens = s.split(" ");
		if (tokens.length != 4){
		    throw(new IllegalArgumentException("File has invalid format."));
		}
		num_of_vertices  = Integer.parseInt(tokens[2].trim());
		declared_edges  = Integer.parseInt(tokens[3].trim());
	    
		degree = new int[num_of_vertices];
		adjacency_list = (ArrayList<Integer>[]) new ArrayList[num_of_vertices];
		for (int i=0; i<num_of_vertices; i++)
		    {
			adjacency_list[i] = new ArrayList<Integer>();
			degree[i]=0;
		    }

		adjacency_matrix = new int[num_of_vertices][num_of_vertices];		
		for (int e=0; e<num_of_vertices; e++)
		    {
		    for (int e1=0; e1<num_of_vertices; e1++)
			adjacency_matrix[e][e1]=0;
		    }
		continue;
            }
            if(s.matches("e.*")){
		s = s.trim();
		String[] tokens = s.split(" ");
		if (tokens.length != 3){
		    throw(new IllegalArgumentException("File has invalid format."));
		}

		int v = Integer.parseInt(tokens[1]) - 1;
		int w = Integer.parseInt(tokens[2]) - 1;
		if (adjacency_matrix[v][w] == 0) {
			num_of_edges++;
			// if (v > w) {
			// 	int h = v;
			// 	v = w;
			// 	w = h;
			// }
			// pair<Vertex, Vertex> edge;
			// edge.first = v;
			// edge.second = w;
			// edges.push_back(edge);
		}
		adjacency_matrix[v][w] = adjacency_matrix[w][v] = 1;

                continue;


            }
	   
            if(s.matches("c.*")){
                continue;
            }
	    if(s.matches(" .*")){
                continue;
            }
	    if(s.matches("\n")){
                continue;
            }
            throw(new IllegalArgumentException("File has invalid format."));
        }
        
        for(String s = reader.readLine();s!=null;s=reader.readLine()){
            if (s.equals("EOF")){
                break;
            }
        }


	if (num_of_edges != declared_edges)
	    {
		System.out.printf("%d %d\n",num_of_edges,declared_edges);System.out.flush(); 
		//		throw(new IllegalArgumentException("File has not as many edges as declared."));
	    }

	// Organize some data
	//updating vector degree
	av_degree = 0;
	max_degree = 0;
	min_degree = VERTEX_MAX;
	for (int i = 0; i < num_of_vertices; i++) {
	    //adjacency_list[i].clear();
	    //	degree[i] = 0;
		for (int j = 0; j < num_of_vertices; j++) {
			if (Edge(i, j)) {
				degree[i]++;
				adjacency_list[i].add(j);
			}
		}
		if (degree[i] > max_degree)
			max_degree = degree[i];
		if (degree[i] < min_degree)
			min_degree = degree[i];

		av_degree += degree[i];
	}

	av_degree /= num_of_vertices;
	density = ((float) (num_of_edges)) / (((float) (num_of_vertices) / 2)
			* (num_of_vertices - 1));

	if (print_statistics) {
		for (int i = 0; i < num_of_vertices; i++) {
			//cout<<(double) degree[i]/(num_of_vertices-1)<<" ";
		}
		//cout<<endl;
		double av_degree_norm = av_degree / (num_of_vertices - 1);
		double std_degree;
		double sum_sd = 0;
		for (int i = 0; i < num_of_vertices; i++) {
			sum_sd += (degree[i] - av_degree) * (degree[i] - av_degree);
		}
		std_degree = Math.sqrt(sum_sd / num_of_vertices);

		System.out.printf("\nx %s %d %.4f %.1f %.1f %.4f %.4f %d %d\n", 
				  (new File(filename)).getName(),
				num_of_vertices, density, av_degree, std_degree,
				av_degree_norm, (std_degree / (num_of_vertices - 1)),
				min_degree, max_degree);
		System.exit(0);
	}

    }

}
	
