import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;


public class Problem {
    double[][] coordinates = null;
    long[][] distance = null;
    int n_of_cities;
    
    public Problem() {
        //Dont allow the default constructor.
    }
    
    /**
     * Reads the coordinates from a file and constructs the Problem.
     * 
     * @param filename 
     * @throws FileNotFoundException
     * @throws IOException
     */
    public Problem( String filename) throws FileNotFoundException, IOException{
        loadFromFile(filename);
    }
    
    /**
     * Load coordinates from a file into this Problem.
     * 
     * @param filename
     * @throws FileNotFoundException
     * @throws IOException
     */
    public void loadFromFile(String filename) throws FileNotFoundException, IOException{
        BufferedReader reader = new BufferedReader(new FileReader(filename));
        
        for(String s = reader.readLine();s!=null;s=reader.readLine()){
            if(s.matches("NAME.*")){
                continue;
            }
            if(s.matches("COMMENT.*")){
                continue;
            }
            if(s.matches("TYPE.*")){
                continue;
            }
            if(s.matches("DIMENSION.*")){
                String[] tokens = s.split(":");
                int dim = Integer.parseInt(tokens[1].trim());
                n_of_cities=dim;
                coordinates = new double[dim][2];
                distance = new long[dim][dim];
                continue;
            }
            if(s.matches("EDGE_WEIGHT_TYPE.*")){
                continue;
            }
            if(s.matches("NODE_COORD_SECTION")){
                break;
            }
            if(s.matches("FIXED_EDGES_SECTION")){
                while(!reader.readLine().trim().equals("-1"));
                continue;
            }
            if (s.matches("DISPLAY_DATA_TYPE.*")) {
        	continue;
            }

            throw(new IllegalArgumentException("File has invalid format."+s));
        }
        
        for(String s = reader.readLine();s!=null;s=reader.readLine()){
            s = s.trim();
            if (s.equals("EOF")){
                break;
            }
            String[] tokens = s.split(" +");
            if (tokens.length != 3){
                throw(new IllegalArgumentException("File has invalid format."));
            }
            int index = Integer.parseInt(tokens[0]);
            double x = Double.parseDouble(tokens[1]);
            double y = Double.parseDouble(tokens[2]);
            coordinates[index-1][0] = x;
            coordinates[index-1][1] = y;
        }
        //
        //Rounds the distance in order to emulate the tsplib c++ implementation..
        for (int i=0;i<n_of_cities;i++) {
            for (int j=i;j<n_of_cities;j++) {
            distance[i][j]=distance[j][i]=Math.round(Math.sqrt(Math.pow(coordinates[i][0]-coordinates[j][0],2)
        	    				+Math.pow(coordinates[i][1]-coordinates[j][1],2)));
            }
        }
        }	
    
    /** 
     * Get coordinates of the point with the given key.
     * @param key
     * @return
     */
    public double[] getCoords(int key){
        double[] coords = coordinates[key].clone();
        return coords;
    }
    public double[][] getAllCoords(){
        return coordinates.clone();
    }
    
    /**
     * Returns the number of coordinates in the set, including deleted ones.
     * @return
     */
    public int getSize(){
        return n_of_cities;
    }
    
    
    /**
     * Return the complete length of the tour, the indexes of which are given
     * in the array tour.
     * 
     * @param tour
     * @return The length of this tour
     */
    public long tourValue(int[] tour){
        long result = 0;
        for(int i = 0;i<tour.length-1;i++){
            result+=distance(tour[i],tour[i+1]);
        }
        result += distance(tour[0],tour[tour.length-1]);
        return result;
    }
    
    /**
     * Returns the distance between the coordinates with key point1 and key point2,
     * rounded to the nearest integer.
     * @param point1 Key of the first point
     * @param point2 Key of the second point
     * @return
     */
    public long distance(int point1, int point2){
	return distance[point1][point2];
    }	
    
    

}
